package be.iminds.ilabt.jfed.ui.cli2.instruction;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * User: instructions on which user account to use
 *
 * There are some options how this is specified:
 *   - 1 login PEM file with cert and key
 *   - 2 login PEM files, once with cert and one with key
 *   ( - PEM directly in yml ?)
 *
 * The password also needs to be specified (if present).
 * There are some way to do this
 *   - on command line or in yml file (most insecure)
 *   - read from file or stdin (only slighlty less insecure (perhaps))
 *   - interactivly in terminal (default)
 *
 * Speaksfor is also configured here
 */
public class User {
    public enum PasswordMethod { NONE, INTERACTIVE, DIRECT, FILE, STDIN }

    /** allowed to be multiple (which are just merged), but typically just one */
    @Nonnull private final List<String> pem;

    @Nonnull private final PasswordMethod passwordMethod;
    @Nullable private final String password;

    @Nonnull private final List<String> speaksForCredential;

    @JsonCreator
    public User(@JsonProperty("pem") @Nullable List<String> pem,
                @JsonProperty("passwordMethod") @Nullable PasswordMethod passwordMethod,
                @JsonProperty("password") @Nullable String password,
                @JsonProperty("speaksForCredential") @Nullable List<String> speaksForCredential) {
        this.pem = pem == null ? Collections.emptyList() :
                Collections.unmodifiableList(new ArrayList<>(pem));
        this.passwordMethod = passwordMethod == null ? PasswordMethod.INTERACTIVE : passwordMethod;
        this.password = password;
        this.speaksForCredential = speaksForCredential == null ? Collections.emptyList() :
                Collections.unmodifiableList(new ArrayList<>(speaksForCredential));;
    }

    public User() {
        this.pem = Collections.emptyList();
        this.passwordMethod = PasswordMethod.INTERACTIVE;
        this.password = null;
        this.speaksForCredential = Collections.emptyList();
    }

    @JsonProperty
    @Nonnull
    public List<String> getPem() {
        return pem;
    }

    @JsonProperty
    @Nonnull
    public PasswordMethod getPasswordMethod() {
        return passwordMethod;
    }

    @JsonProperty
    @Nullable
    public String getPassword() {
        return password;
    }

    @JsonProperty
    @Nonnull
    public List<String> getSpeaksForCredential() {
        return speaksForCredential;
    }
}
